<?php
namespace Deargonaut\PaypalCsvToMt940\Models;

class Mutation
{

    const STATUS_COMPLETED = 1;
    const STATUS_INCOMPLETE = 2;

    public  $amount = 0.00,
            $currency,
            $date,
            $description,
            $balance = 0.00,
            $purposes = [],
            $transaction,
            $status;


    public function getDate()
    {
        return date('r', strtotime($this->date ?: time()));
    }

    public function getDescription()
    {
        return $this->description ?: '';
    }

    public function getPurposeText()
    {
        if (count($this->purposes) > 0)
        {
            uasort($this->purposes, array('Deargonaut\PaypalCsvToMt940\Models\Purpose', 'cmp'));
            $txt = '';
            foreach ($this->purposes as $purp)
            {
                $txt .= strval($purp);
            }

            return $txt;
        }
        else
        {
            return $this->getDescription();
        }
    }

    public function addPurpose($purpose)
    {
        if (is_array($purpose))
        {
            $this->purposes = array_merge($this->purposes, $purpose);
        }
        else
        {
            $this->purposes[] = $purpose;
        }
    }

    public function setTransaction($transaction)
    {
        $this->transaction = $transaction;
    }

    public function getTransaction()
    {
        return $this->transaction;
    }

    /**
     * Parse a string to a float value.
     * 
     * @param string $amount CSV mutations amount text.
     * 
     * @return float
     */
    public static function parseToFloat($amount)
    {
        $parsedAmount = 0.0;
        if(strstr($amount, ','))
        {
            if(strstr($amount, '.'))
            {
                $exp = explode(',', $amount);
                if(strstr($exp[0], '.'))
                {
                    $parsedAmount = (float) str_replace(',', '.', $amount);
                    $parsedAmount = (float) str_replace('.', '', $amount);
                }
                else if(strstr($exp[1], '.'))
                {
                    $parsedAmount = (float) str_replace(',', '', $amount);
                }
                else if(count($exp) > 2)
                {
                    $parsedAmount = (float) str_replace(',', '', $amount);
                }
            }
            else {
                $parsedAmount = (float) str_replace(',', '.', $amount);
            }
        }

        return $parsedAmount;
    }

    public function setAmount($amount)
    {
        $this->amount = $this::parseToFloat($amount);
    }

    public function setBalance($amount)
    {
        $this->balance = $this::parseToFloat($amount);
    }

    public function setCurrency($currency)
    {
        $this->currency = $currency;
    }

    public function setStatus($status)
    {
        if (!in_array($status, [$this::STATUS_COMPLETED, $this::STATUS_INCOMPLETE]))
        {
            throw new \Exception("Unknown mutation status passed!");
        }
        $this->status = $status;
    }
}