<?php
namespace Deargonaut\PaypalCsvToMt940\Models;

class Purpose
{

    const FLAG_CODE = '';
    const FLAG_BOOK_TXT = 0;
    const FLAG_PRIMA = 10;
    const FLAG_PP_1 = 20;
    const FLAG_PP_2 = 21;
    const FLAG_PP_3 = 22;
    const FLAG_PP_4 = 23;
    const FLAG_PP_5 = 24;
    const FLAG_PP_6 = 25;
    const FLAG_PP_7 = 26;
    const FLAG_PP_8 = 27;
    const FLAG_PP_9 = 28;
    const FLAG_PP_10 = 29;
    const FLAG_BLZ = 30;
    const FLAG_KTNR = 31;
    const FLAG_NAME_1 = 32;
    const FLAG_NAME_2 = 33;
    const FLAG_TXTKEYEXT = 34;
    const FLAG_PP_11 = 60;
    const FLAG_PP_12 = 61;
    const FLAG_PP_13 = 62;
    const FLAG_PP_14 = 63;

    public $flag;
    public $value;

    public function __construct($flag, $value)
    {
        $this->setFlag($flag);
        $this->setValue($value);
    }

    public static function cmp($a, $b)
    {
        if ($a->flag == $b->flag)
        {
            return 0;
        }

        return ($a->flag < $b->flag) ? -1 : 1;
    }

    public function setFlag($flag)
    {
        $this->flag = $flag;
    }

    public function setValue($value)
    {
        $this->value = $value;
    }

    public function __toString()
    {
        return ($this->flag === self::FLAG_CODE ? '' : '?' . sprintf("%'02d", $this->flag)) . $this->value;
    }

    public static function generateGeneric($flags, $names)
    {
        $i = 0;
        $elements = [];

        foreach($names as $value)
        {
            if($i < count($flags))
            {
                $elements[] = new Purpose($flags[$i], $value);
                $i++;
            }
        }
        return $elements;
    }

    public static function generatePurposeText($text)
    {
        $flags = [
            self::FLAG_PP_1, self::FLAG_PP_2, self::FLAG_PP_3, self::FLAG_PP_4, self::FLAG_PP_5, self::FLAG_PP_6,
            self::FLAG_PP_7, self::FLAG_PP_8, self::FLAG_PP_9, self::FLAG_PP_10, self::FLAG_PP_11, self::FLAG_PP_12,
            self::FLAG_PP_13, self::FLAG_PP_14
        ];
        $i = 0;
        $elements = [];

        foreach(self::unicodeStringSplit($text, 27) as $line)
        {
            if($i < count($flags))
            {
                $elements[] = new Purpose($flags[$i], $line);
                $i++;
            }
        }

        return $elements;
    }

    /**
     * Splits a long unicode text into chunks.
     * 
     * @author qeremy [atta] gmail [dotta] com
     * @see https://www.php.net/manual/en/function.str-split.php
     * 
     * @param string $str Unicode string to be splitted
     * @param int    $length Max. chunk size
     * @param int    $maxLines Spec. max. amount of array entries. 0 = endless.
     * 
     * @return string[] Chunks or false if a split is not possibl.
     */
    public static function unicodeStringSplit($str, $length = 1, $maxLines = 0)
    {
        $tmp = preg_split('~~u', $str, -1, PREG_SPLIT_NO_EMPTY);
        if ($length > 1) {
            $chunks = array_chunk($tmp, $length);
            foreach ($chunks as $i => $chunk) {
                $chunks[$i] = join('', (array) $chunk);
            }
            $tmp = $chunks;
        }

        // Drop too much lines if requested.
        if ($tmp !== false && $maxLines > 0 && count($tmp) > $maxLines) {
            $tmp = array_slice($tmp, 0, $maxLines, true);
        }
        return $tmp;
    }
}